#!/bin/bash

set -e

# /boot/efi 512M
# /boot     1024M
# /         6
# /backup   free/9
# /data     4
# swap      ram*1.2

if [ "${is_efi}" == "false" ]; then
    return
fi

### efi
efi=512


### boot
if [[ "${isluks_lvm}" == "true" ]]; then
        boot=2048
else
        boot=1024
fi

disk=$(get_value devpath)
size=$(fdisk -s "${disk}")
real_size=$(( size / 1024 ))


### backup
backup_size=$(( real_size / 9 ))
if [ $backup_size -lt 16384 ]; then
    backup_size=16384
elif [ $backup_size -gt 102400 ]; then
    backup_size=102400
fi


ram=$(cat < /proc/meminfo  | grep ^MemTotal: | awk '{print $2}')


### swap
swap=$(( ram * 12 / 1024 / 10 ))
if [ $swap -gt 64000 ]; then
    swap=64000
fi
swapfile=0
if [ x${is_swapfile} == x"true" ]; then
    swapfile=${swap}
    swap=0
fi
free_size=$(( real_size - efi - boot - backup_size - swap ))


### root
root_size=$(( free_size * 6 / 10 ))
if [ $root_size -lt 16384 ]; then
    root_size=16384
    if [ $free_size -lt 0 ]; then
        exit 1
    fi
elif [ $root_size -gt 102400 ]; then
    root_size=102400
fi
root_size=$((root_size + swapfile))


### data
data_size=$(( free_size - root_size ))
if [ $data_size -lt 0 ]; then
    exit 1
fi



echo "efi: $efi"
echo "boot: $boot"
echo "root_size: $root_size"
echo "backup_size: $backup_size"
echo "data_size: $data_size"
echo "swap: $swap"
echo "swapfile: $swapfile"
