#!/bin/bash

set -e
set -o pipefail

# TODO, 硬编码
# 拷贝配置文件
do_copy_config() {
  cp utils /tmp

  if grep -q automatic /proc/cmdline; then
    [[ -f /cdrom/ky-installer.cfg ]] && cp /cdrom/ky-installer.cfg /tmp
  elif grep -qE 'ple-mode|test-mode' /proc/cmdline; then
    [[ -f /cdrom/kyple-installer.cfg ]] && cp /cdrom/kyple-installer.cfg /tmp/ky-installer.cfg
  else
    cp /usr/share/kylin-os-installer/ky-installer.cfg /tmp
    #cp /home/kylin/kylin-os-installer/ky-installer.cfg /tmp
  fi
}

do_copy_config

. /tmp/utils

# 运行 prepare 脚本
do_prepare() {
  mkdir -p /var/log/installer

  run_scripts_dir prepare

  if [[ -d /cdrom/hooks/prepare ]]; then
      run_scripts_hooks /cdrom/hooks/prepare
  fi
}

do_run_target() {
  mkdir -p /target/${IN_TARGET}
  cp utils /tmp/ky-installer.cfg /target/tmp
  cp -r k.sh in_chroot /target/${IN_TARGET}

  chroot /target bash ${IN_TARGET}/k.sh
}

# 运行 after 脚本
do_after() {
  run_scripts_dir after

  if [[ -d /cdrom/hooks/after ]]; then
      run_scripts_hooks /cdrom/hooks/after
  fi
}

# 执行 post 脚本
do_cdrom_post_actions() {
    if [[ -f /cdrom/.kylin-post-actions ]]; then
        cp /cdrom/.kylin-post-actions /target/${IN_TARGET}
        chroot /target bash ${IN_TARGET}/.kylin-post-actions >>${LOG_FILE} 2>&1
    fi
}

# 拷贝日志
do_copy_log() {
    msg "正在拷贝安装日志..."
    cat ${LOG_FILE} /target/${LOG_FILE} >/target/var/log/installer/kylin-os-installer-script.log
}

# 出厂备份
do_factory_backup() {
    umount /target/backup
    msg "正在进行系统备份..."
    /usr/bin/backup-auto --factorybackup /target /backup || true
    msg "系统备份完成"
}

do_install_success() {
    if [[ x${is_990_9a0} = x"true" ]]; then
        mkdir -p /target/boot/efi
        touch /target/boot/efi/INSTALL_SUCCESS
    fi
}

handle_factory_backup() {
  factory_backup=$(get_value factory-backup)
  if [[ -f /target/etc/.bootinfo ]]; then
      if grep -q 'factory-backup' /proc/cmdline; then
          do_factory_backup
      elif [[ $factory_backup -eq 1 ]]; then
          do_factory_backup
      fi
  fi
}

do_finished() {
  echo 'finished'
  touch /tmp/z-finished-install
}

# 运行 prepare 阶段脚本
do_prepare

# 执行 target inchroot 脚本
do_run_target

# 运行 after 阶段脚本
do_after

# 执行 postactions 脚本
do_cdrom_post_actions

# 拷贝安装日志
do_copy_log

do_install_success

# 处理出厂备份
handle_factory_backup

# 卸载文件系统
umount_all

do_finished
